@@ -15,6 +15,15 @@ if is_service_enabled tacker; then
         echo_summary "Installing Tacker"
         install_tacker
 
+        if use_library_from_git heat-translator; then
+            git_clone_by_name heat-translator
+            setup_dev_lib heat-translator
+        fi
+        if use_library_from_git tosca-parser; then
+            git_clone_by_name tosca-parser
+            setup_dev_lib tosca-parser
+        fi
+
     elif [[ "$1" == "stack" && "$2" == "post-config" ]]; then
         # Configure after the other layer 1 and 2 services have been configured
         echo_summary "Configuring Tacker"
@@ -32,15 +41,13 @@ if is_service_enabled tacker; then
             tacker_horizon_install
         fi
 
-        if [[ "${TACKER_MODE}" == "all" ]]; then
-            echo_summary "Modifying Heat policy.json file"
-            modify_heat_flavor_policy_rule
+        if [[ "${TACKER_MODE}" == "all" || "${IS_ZUUL_FT}" == "True" ]]; then
             echo_summary "Setup initial tacker network"
             tacker_create_initial_network
             echo_summary "Check and download images for tacker initial"
             tacker_check_and_download_images
-            echo_summary "Registering default VIM"
-            tacker_register_default_vim
+            echo_summary "Setup default VIM resources"
+            tacker_setup_default_vim_resources
 
             if is_service_enabled ceilometer; then
                 echo_summary "Configure maintenance event types"
