@@ -21,7 +21,6 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import importutils
 import routes
-import six
 import webob.dec
 import webob.exc
 
@@ -35,8 +34,7 @@ from tacker import wsgi
 LOG = logging.getLogger(__name__)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class PluginInterface(object):
+class PluginInterface(object, metaclass=abc.ABCMeta):
 
     @classmethod
     def __subclasshook__(cls, klass):
@@ -326,7 +324,7 @@ class ExtensionMiddleware(wsgi.Middleware):
         """Return a dict of ActionExtensionController-s by collection."""
         action_controllers = {}
         for action in ext_mgr.get_actions():
-            if action.collection not in action_controllers.keys():
+            if action.collection not in action_controllers:
                 controller = ActionExtensionController(application)
                 mapper.connect("/%s/:(id)/action.:(format)" %
                                action.collection,
@@ -345,7 +343,7 @@ class ExtensionMiddleware(wsgi.Middleware):
         """Returns a dict of RequestExtensionController-s by collection."""
         request_ext_controllers = {}
         for req_ext in ext_mgr.get_request_extensions():
-            if req_ext.key not in request_ext_controllers.keys():
+            if req_ext.key not in request_ext_controllers:
                 controller = RequestExtensionController(application)
                 mapper.connect(req_ext.url_route + '.:(format)',
                                action='process',
