@@ -267,16 +267,6 @@ class VnfResource(model_base.BASE, models.SoftDeleteMixin,
     resource_status = sa.Column(sa.String(255), nullable=False)
 
 
-class VnfLcmSubscriptions(model_base.BASE, models.SoftDeleteMixin,
-                models.TimestampMixin):
-    """Contains all info about vnf LCM Subscriptions."""
-
-    __tablename__ = 'vnf_lcm_subscriptions'
-    id = sa.Column(sa.String(36), nullable=False, primary_key=True)
-    callback_uri = sa.Column(sa.String(255), nullable=False)
-    subscription_authentication = sa.Column(sa.JSON, nullable=True)
-
-
 class VnfLcmFilters(model_base.BASE):
     """Contains all info about vnf LCM filters."""
 
@@ -287,13 +277,45 @@ class VnfLcmFilters(model_base.BASE):
                             sa.ForeignKey('vnf_lcm_subscriptions.id'),
                             nullable=False)
     filter = sa.Column(sa.JSON, nullable=False)
-    notification_types = sa.Column(sa.VARBINARY(255), nullable=True)
+    vnf_products_from_providers = sa.Column(sa.JSON, nullable=True)
+    vnfd_ids = sa.Column(sa.Text(), nullable=True)
+    vnfd_ids_len = sa.Column(sa.Integer, nullable=True)
+    vnf_provider = sa.Column(sa.Text(), nullable=True)
+    vnf_product_name = sa.Column(sa.Text(), nullable=True)
+    vnf_software_version = sa.Column(sa.Text(), nullable=True)
+    vnfd_versions = sa.Column(sa.Text(), nullable=True)
+    vnfd_versions_len = sa.Column(sa.Integer, nullable=True)
+    vnf_instance_ids = sa.Column(sa.Text(), nullable=True)
+    vnf_instance_ids_len = sa.Column(sa.Integer, nullable=True)
+    vnf_instance_names = sa.Column(sa.Text(), nullable=True)
+    vnf_instance_names_len = sa.Column(sa.Integer, nullable=True)
+    notification_types = sa.Column(
+        sa.LargeBinary(
+            length=__maxsize__),
+        nullable=True)
     notification_types_len = sa.Column(sa.Integer, nullable=True)
     operation_types = sa.Column(
         sa.LargeBinary(
             length=__maxsize__),
         nullable=True)
     operation_types_len = sa.Column(sa.Integer, nullable=True)
+    operation_states = sa.Column(sa.Text(), nullable=True)
+    operation_states_len = sa.Column(sa.Integer, nullable=True)
+
+
+class VnfLcmSubscriptions(model_base.BASE, models.SoftDeleteMixin,
+                models.TimestampMixin):
+    """Contains all info about vnf LCM Subscriptions."""
+
+    __tablename__ = 'vnf_lcm_subscriptions'
+    id = sa.Column(sa.String(36), nullable=False, primary_key=True)
+    callback_uri = sa.Column(sa.String(255), nullable=False)
+    subscription_authentication = sa.Column(sa.JSON, nullable=True)
+
+    subscription_filter = orm.relationship(
+        VnfLcmFilters,
+        primaryjoin='and_(VnfLcmSubscriptions.id == '
+                    'VnfLcmFilters.subscription_uuid)')
 
 
 class VnfLcmOpOccs(model_base.BASE, models.SoftDeleteMixin,
@@ -305,6 +327,7 @@ class VnfLcmOpOccs(model_base.BASE, models.SoftDeleteMixin,
     vnf_instance_id = sa.Column(sa.String(36),
                                 sa.ForeignKey('vnf_instances.id'),
                                 nullable=False)
+    grant_id = sa.Column(sa.String(36), nullable=True)
     state_entered_time = sa.Column(sa.DateTime(), nullable=False)
     start_time = sa.Column(sa.DateTime(), nullable=False)
     operation_state = sa.Column(sa.String(length=255), nullable=False)
@@ -315,6 +338,7 @@ class VnfLcmOpOccs(model_base.BASE, models.SoftDeleteMixin,
     error = sa.Column(sa.JSON(), nullable=True)
     resource_changes = sa.Column(sa.JSON(), nullable=True)
     changed_info = sa.Column(sa.JSON(), nullable=True)
+    changed_ext_connectivity = sa.Column(sa.JSON(), nullable=True)
     error_point = sa.Column(sa.Integer, nullable=False)
 
 
