@@ -15,7 +15,6 @@
 #    under the License.
 
 import os
-import six
 import yaml
 
 from keystoneauth1 import exceptions
@@ -172,9 +171,9 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
         """Validate and set VIM placements."""
 
         if 'key_type' in vim_obj['auth_cred']:
-            vim_obj['auth_cred'].pop(u'key_type')
+            vim_obj['auth_cred'].pop('key_type')
         if 'secret_uuid' in vim_obj['auth_cred']:
-            vim_obj['auth_cred'].pop(u'secret_uuid')
+            vim_obj['auth_cred'].pop('secret_uuid')
 
         ks_client = self.authenticate_vim(vim_obj)
         self.discover_placement_attr(vim_obj, ks_client)
@@ -253,10 +252,7 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
             key_file = os.path.join(CONF.vim_keys.openstack, vim_id)
             try:
                 with open(key_file, 'wb') as f:
-                    if six.PY2:
-                        f.write(fernet_key.decode('utf-8'))
-                    else:
-                        f.write(fernet_key)
+                    f.write(fernet_key)
                     LOG.debug('VIM auth successfully stored for vim %s',
                               vim_id)
             except IOError:
@@ -271,7 +267,7 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
         :param resource_name: name of resource to locate
         :return: ID of resource
         """
-        if resource_type in _VALID_RESOURCE_TYPES.keys():
+        if resource_type in _VALID_RESOURCE_TYPES:
             res_cmd_map = _VALID_RESOURCE_TYPES[resource_type]
             client_type = res_cmd_map['client']
             cmd = res_cmd_map['cmd']
@@ -731,6 +727,7 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
         wg.task(**kwargs)
         if not wg.get_tasks():
             raise nfvo.NoTasksException(resource=resource, action=action)
+        yaml.SafeDumper.ignore_aliases = lambda self, data: True
         definition_yaml = yaml.safe_dump(wg.definition)
         workflow = mistral_client.workflows.create(definition_yaml)
         return {'id': workflow[0].id, 'input': wg.get_input_dict()}
