@@ -81,8 +81,14 @@ class TestUtils(testtools.TestCase):
         }
         vdu_flavor_dict = {'VDU1': {'ram': 'vdu1_flavor_ram_change'}}
         vdu_image_dict = {'VDU1': 'vdu1_image_uuid_change'}
-        cpd_vl_dict = {'CP1': {'cp1_network_id_change_1',
-                               'cp1_network_id_change_2'}}
+        cpd_vl_dict = {
+            'CP1': {
+                'network': {
+                    'cp1_network_id_change_1',
+                    'cp1_network_id_change_2'
+                }
+            }
+        }
         expected_final_param_dict = {
             'nfv': {
                 'CP': {
@@ -129,8 +135,7 @@ class TestUtils(testtools.TestCase):
 
     def test_create_vdu_flavor_dict(self):
         vnfd_dict = self._read_file('vnfd_lcm_user_data.yaml')
-        test_vnfd_dict = {'VNF': {}, 'VDU1':
-                {'ram': 512, 'vcpus': 1, 'disk': 1}, 'CP1': {}}
+        test_vnfd_dict = {'VDU1': {'ram': 512, 'vcpus': 1, 'disk': 1}}
         vdu_flavor_dict = utils.create_vdu_flavor_dict(vnfd_dict)
         self.assertEqual(test_vnfd_dict, vdu_flavor_dict)
 
