@@ -499,7 +499,7 @@ class TestVNFMPlugin(db_base.SqlTestCase):
         self._insert_dummy_vnf_template()
         vnf_obj = utils.get_dummy_vnf_obj()
         vnf_obj['vnf']['attributes'] = {'param_values':
-        {'image_name': 'cirros-0.4.0-x86_64-disk', 'flavor': 'm1.tiny'}}
+        {'image_name': 'cirros-0.5.2-x86_64-disk', 'flavor': 'm1.tiny'}}
         result = self.vnfm_plugin.create_vnf(self.context, vnf_obj)
         self.assertIsNotNone(result)
         self.assertEqual(vnf_obj['vnf']['attributes']['param_values'],
@@ -703,20 +703,20 @@ class TestVNFMPlugin(db_base.SqlTestCase):
     def _insert_dummy_ns_template(self):
         session = self.context.session
         attributes = {
-            u'nsd': 'imports: [VNF1, VNF2]\ntopology_template:\n  inputs:\n  '
-                    '  vl1_name: {default: net_mgmt, description: name of VL1'
-                    ' virtuallink, type: string}\n    vl2_name: {default: '
-                    'net0, description: name of VL2 virtuallink, type: string'
-                    '}\n  node_templates:\n    VL1:\n      properties:\n     '
-                    '   network_name: {get_input: vl1_name}\n        vendor: '
-                    'tacker\n      type: tosca.nodes.nfv.VL\n    VL2:\n      '
-                    'properties:\n        network_name: {get_input: vl2_name}'
-                    '\n        vendor: tacker\n      type: tosca.nodes.nfv.VL'
-                    '\n    VNF1:\n      requirements:\n      - {virtualLink1: '
-                    'VL1}\n      - {virtualLink2: VL2}\n      type: tosca.node'
-                    's.nfv.VNF1\n    VNF2: {type: tosca.nodes.nfv.VNF2}\ntosca'
-                    '_definitions_version: tosca_simple_profile_for_nfv_1_0_0'
-                    '\n'}
+            'nsd': 'imports: [VNF1, VNF2]\ntopology_template:\n  inputs:\n  '
+                   '  vl1_name: {default: net_mgmt, description: name of VL1'
+                   ' virtuallink, type: string}\n    vl2_name: {default: '
+                   'net0, description: name of VL2 virtuallink, type: string'
+                   '}\n  node_templates:\n    VL1:\n      properties:\n     '
+                   '   network_name: {get_input: vl1_name}\n        vendor: '
+                   'tacker\n      type: tosca.nodes.nfv.VL\n    VL2:\n      '
+                   'properties:\n        network_name: {get_input: vl2_name}'
+                   '\n        vendor: tacker\n      type: tosca.nodes.nfv.VL'
+                   '\n    VNF1:\n      requirements:\n      - {virtualLink1: '
+                   'VL1}\n      - {virtualLink2: VL2}\n      type: tosca.node'
+                   's.nfv.VNF1\n    VNF2: {type: tosca.nodes.nfv.VNF2}\ntosca'
+                   '_definitions_version: tosca_simple_profile_for_nfv_1_0_0'
+                   '\n'}
         nsd_template = ns_db.NSD(
             id='eb094833-995e-49f0-a047-dfb56aaf7c4e',
             tenant_id='ad7ebc56538745a08ef7c5e97f8bd437',
@@ -1142,7 +1142,7 @@ class TestVNFMPlugin(db_base.SqlTestCase):
         heal_request_data_obj = heal_vnf_request.HealVnfRequest(
             stack_id=dummy_device_obj['instance_id'],
             cause='VNF monitoring fails.',
-            additional_params=[additional_params_obj])
+            legacy_additional_params=[additional_params_obj])
         result = self.vnfm_plugin.heal_vnf(self.context,
                                            dummy_device_obj['id'],
                                            heal_request_data_obj)
