@@ -17,8 +17,12 @@
 # key
 KEY_ACTION = 'action'
 KEY_KWARGS = 'kwargs'
+KEY_SCALE = 'scale_stack_id'
 
 # ACTION type
 ACTION_CREATE_VNF = 'create_vnf'
 ACTION_UPDATE_VNF = 'update_vnf'
 ACTION_DELETE_VNF = 'delete_vnf'
+ACTION_HEAL_VNF = 'heal_vnf'
+ACTION_SCALE_IN_VNF = 'scale_in_vnf'
+ACTION_SCALE_OUT_VNF = 'scale_out_vnf'
