@@ -15,16 +15,17 @@
 #    under the License.
 
 import inspect
-import six
 import yaml
 
 import eventlet
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_serialization import jsonutils
 from oslo_utils import excutils
 from oslo_utils import uuidutils
 from toscaparser.tosca_template import ToscaTemplate
 
+from tacker._i18n import _
 from tacker.api.v1 import attributes
 from tacker.common import driver_manager
 from tacker.common import exceptions
@@ -33,6 +34,7 @@ from tacker import context as t_context
 from tacker.db.vnfm import vnfm_db
 from tacker.extensions import vnfm
 from tacker.plugins.common import constants
+from tacker.plugins import fenix
 from tacker.tosca import utils as toscautils
 from tacker.vnfm.mgmt_drivers import constants as mgmt_constants
 from tacker.vnfm import monitor
@@ -114,6 +116,7 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
 
     Plugin which supports Tacker framework
     """
+
     OPTS_INFRA_DRIVER = [
         cfg.ListOpt(
             'infra_driver', default=['noop', 'openstack', 'kubernetes'],
@@ -145,7 +148,9 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
         self._vnf_monitor = monitor.VNFMonitor(self.boot_wait)
         self._vnf_alarm_monitor = monitor.VNFAlarmMonitor()
         self._vnf_reservation_monitor = monitor.VNFReservationAlarmMonitor()
+        self._vnf_maintenance_monitor = monitor.VNFMaintenanceAlarmMonitor()
         self._vnf_app_monitor = monitor.VNFAppMonitor()
+        self._vnf_maintenance_plugin = fenix.FenixPlugin()
         self._init_monitoring()
 
     def _init_monitoring(self):
@@ -256,6 +261,13 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
         vnfd_dict = yaml.safe_load(vnfd_yaml)
         if not (vnfd_dict and vnfd_dict.get('tosca_definitions_version')):
             return
+        try:
+            toscautils.updateimports(vnfd_dict)
+            tosca_vnfd = ToscaTemplate(a_file=False,
+                                  yaml_dict_tpl=vnfd_dict)
+        except Exception as e:
+            LOG.exception("tosca-parser error: %s", str(e))
+            raise vnfm.ToscaParserFailed(error_msg_details=str(e))
         polices = vnfd_dict['topology_template'].get('policies', [])
         for policy_dict in polices:
             name, policy = list(policy_dict.items())[0]
@@ -271,6 +283,13 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
                         self, context, vnf_dict, policy)
                 vnf_dict['attributes']['reservation_policy'] = vnf_dict['id']
                 vnf_dict['attributes'].update(alarm_url)
+        maintenance_vdus = toscautils.find_maintenance_vdus(tosca_vnfd)
+        maintenance = \
+            self._vnf_maintenance_monitor.update_vnf_with_maintenance(
+                vnf_dict, maintenance_vdus)
+        vnf_dict['attributes'].update({
+            'maintenance': jsonutils.dumps(maintenance['vdus'])})
+        vnf_dict['attributes']['maintenance_url'] = maintenance['url']
 
     def add_vnf_to_appmonitor(self, context, vnf_dict):
         appmonitor = self._vnf_app_monitor.create_app_dict(context, vnf_dict)
@@ -279,6 +298,8 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
     def config_vnf(self, context, vnf_dict):
         config = vnf_dict['attributes'].get('config')
         if not config:
+            self._vnf_maintenance_plugin.create_vnf_constraints(self, context,
+                                                                vnf_dict)
             return
         if isinstance(config, str):
             # TODO(dkushwaha) remove this load once db supports storing
@@ -314,8 +335,7 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
             LOG.error("VNF Create failed for vnf_id %s", vnf_id)
             create_failed = True
             vnf_dict['status'] = constants.ERROR
-            self.set_vnf_error_status_reason(context, vnf_id,
-                                             six.text_type(e))
+            self.set_vnf_error_status_reason(context, vnf_id, str(e))
 
         if instance_id is None or create_failed:
             mgmt_ip_address = None
@@ -365,6 +385,7 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
         if driver_name == 'openstack':
             self.mgmt_create_pre(context, vnf_dict)
             self.add_alarm_url_to_vnf(context, vnf_dict)
+        vnf_dict['attributes']['maintenance_group'] = uuidutils.generate_uuid()
 
         try:
             instance_id = self._vnf_manager.invoke(
@@ -375,7 +396,7 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
                       'so delete this vnf',
                       vnf_dict['id'])
             with excutils.save_and_reraise_exception():
-                self.delete_vnf(context, vnf_id)
+                self._delete_vnf(context, vnf_id, force_delete=True)
 
         vnf_dict['instance_id'] = instance_id
         return vnf_dict
@@ -429,9 +450,9 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
 
             if 'app_monitoring_policy' in vnf_dict['attributes']:
                 self.add_vnf_to_appmonitor(context, vnf_dict)
-
             if vnf_dict['status'] is not constants.ERROR:
                 self.add_vnf_to_monitor(context, vnf_dict)
+
             self.config_vnf(context, vnf_dict)
         self.spawn_n(create_vnf_wait)
         return vnf_dict
@@ -444,8 +465,54 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
         self._create_vnf_wait(context, vnf_dict, vim_auth, infra_driver)
         return vnf_dict
 
-    def _update_vnf_wait(self, context, vnf_dict, vim_auth, driver_name,
-                         vnf_heal=False):
+    def _heal_vnf_wait(self, context, vnf_dict, vim_auth, driver_name):
+        kwargs = {
+            mgmt_constants.KEY_ACTION: mgmt_constants.ACTION_HEAL_VNF,
+            mgmt_constants.KEY_KWARGS: {'vnf': vnf_dict},
+        }
+        new_status = constants.ACTIVE
+        placement_attr = vnf_dict['placement_attr']
+        region_name = placement_attr.get('region_name')
+
+        try:
+            self._vnf_manager.invoke(
+                driver_name, 'heal_wait', plugin=self,
+                context=context, vnf_dict=vnf_dict, auth_attr=vim_auth,
+                region_name=region_name)
+            self.mgmt_call(context, vnf_dict, kwargs)
+
+        except vnfm.VNFHealWaitFailed as e:
+            with excutils.save_and_reraise_exception():
+                new_status = constants.ERROR
+                self._vnf_monitor.delete_hosting_vnf(vnf_dict['id'])
+                self._vnf_maintenance_plugin.post(context, vnf_dict)
+                self.set_vnf_error_status_reason(context, vnf_dict['id'],
+                                                 str(e))
+        except exceptions.MgmtDriverException as e:
+            LOG.error('VNF configuration failed')
+            new_status = constants.ERROR
+            self._vnf_monitor.delete_hosting_vnf(vnf_dict['id'])
+            self._vnf_maintenance_plugin.post(context, vnf_dict)
+            self.set_vnf_error_status_reason(context, vnf_dict['id'], str(e))
+
+        del vnf_dict['heal_stack_id']
+        vnf_dict['status'] = new_status
+        self.mgmt_update_post(context, vnf_dict)
+
+        # Update vnf status to 'ACTIVE' so that monitoring can be resumed.
+        evt_details = ("Ends the heal vnf request for VNF '%s'" %
+                       vnf_dict['id'])
+        self._vnf_monitor.update_hosting_vnf(vnf_dict, evt_details)
+        self._vnf_maintenance_plugin.update_vnf_instances(self, context,
+                                                          vnf_dict)
+        self._vnf_maintenance_plugin.post(context, vnf_dict)
+        # _update_vnf_post() method updates vnf_status and mgmt_ip_address
+        self._update_vnf_post(context, vnf_dict['id'],
+                              new_status, vnf_dict,
+                              constants.PENDING_HEAL,
+                              constants.RES_EVT_HEAL)
+
+    def _update_vnf_wait(self, context, vnf_dict, vim_auth, driver_name):
         kwargs = {
             mgmt_constants.KEY_ACTION: mgmt_constants.ACTION_UPDATE_VNF,
             mgmt_constants.KEY_KWARGS: {'vnf': vnf_dict},
@@ -459,37 +526,26 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
                 driver_name, 'update_wait', plugin=self,
                 context=context, vnf_dict=vnf_dict, auth_attr=vim_auth,
                 region_name=region_name)
+
             self.mgmt_call(context, vnf_dict, kwargs)
         except vnfm.VNFUpdateWaitFailed as e:
             with excutils.save_and_reraise_exception():
                 new_status = constants.ERROR
                 self._vnf_monitor.delete_hosting_vnf(vnf_dict['id'])
                 self.set_vnf_error_status_reason(context, vnf_dict['id'],
-                                                 six.text_type(e))
+                                                 str(e))
         except exceptions.MgmtDriverException as e:
             LOG.error('VNF configuration failed')
             new_status = constants.ERROR
             self._vnf_monitor.delete_hosting_vnf(vnf_dict['id'])
-            self.set_vnf_error_status_reason(context, vnf_dict['id'],
-                                             six.text_type(e))
+            self.set_vnf_error_status_reason(context, vnf_dict['id'], str(e))
         vnf_dict['status'] = new_status
         self.mgmt_update_post(context, vnf_dict)
-
-        if vnf_heal:
-            # Update vnf status to 'ACTIVE' so that monitoring can be resumed.
-            evt_details = ("Ends the heal vnf request for VNF '%s'" %
-                           vnf_dict['id'])
-            self._vnf_monitor.update_hosting_vnf(vnf_dict, evt_details)
-            # _update_vnf_post() method updates vnf_status and mgmt_ip_address
-            self._update_vnf_post(context, vnf_dict['id'],
-                                  new_status, vnf_dict,
-                                  constants.PENDING_HEAL,
-                                  constants.RES_EVT_HEAL)
-
-        else:
-            self._update_vnf_post(context, vnf_dict['id'], new_status,
-                                  vnf_dict, constants.PENDING_UPDATE,
-                                  constants.RES_EVT_UPDATE)
+        self._update_vnf_post(context, vnf_dict['id'], new_status,
+                              vnf_dict, constants.PENDING_UPDATE,
+                              constants.RES_EVT_UPDATE)
+        self._vnf_maintenance_plugin.create_vnf_constraints(self, context,
+                                                            vnf_dict)
 
     def update_vnf(self, context, vnf_id, vnf):
         vnf_attributes = vnf['vnf']['attributes']
@@ -502,6 +558,17 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
                 vnf_attributes['config'] = yaml.safe_dump(config)
             else:
                 raise vnfm.InvalidAPIAttributeType(atype=type(config))
+
+        if vnf_attributes.get('param_values'):
+            param = vnf_attributes['param_values']
+            if isinstance(param, dict):
+                # TODO(sripriya) remove this yaml dump once db supports storing
+                # json format of yaml files in a separate column instead of
+                #  key value string pairs in vnf attributes table
+                vnf_attributes['param_values'] = yaml.safe_dump(param)
+            else:
+                raise vnfm.InvalidAPIAttributeType(atype=type(param))
+
         vnf_dict = self._update_vnf_pre(context, vnf_id,
                                         constants.PENDING_UPDATE)
         driver_name, vim_auth = self._get_infra_driver(context, vnf_dict)
@@ -513,13 +580,19 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
                 driver_name, 'update', plugin=self, context=context,
                 vnf_id=instance_id, vnf_dict=vnf_dict,
                 vnf=vnf, auth_attr=vim_auth)
+        except vnfm.VNFUpdateInvalidInput:
+            with excutils.save_and_reraise_exception():
+                vnf_dict['status'] = constants.ACTIVE
+                self._update_vnf_post(context, vnf_id,
+                                      constants.ACTIVE,
+                                      vnf_dict, constants.PENDING_UPDATE,
+                                      constants.RES_EVT_UPDATE)
         except Exception as e:
             with excutils.save_and_reraise_exception():
                 vnf_dict['status'] = constants.ERROR
                 self._vnf_monitor.delete_hosting_vnf(vnf_id)
-                self.set_vnf_error_status_reason(context,
-                                                 vnf_dict['id'],
-                                                 six.text_type(e))
+                self.set_vnf_error_status_reason(context, vnf_dict['id'],
+                                                 str(e))
                 self.mgmt_update_post(context, vnf_dict)
                 self._update_vnf_post(context, vnf_id,
                                       constants.ERROR,
@@ -542,6 +615,9 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
         self._vnf_monitor.update_hosting_vnf(vnf_dict, evt_details)
 
         try:
+            vnf_dict['heal_stack_id'] = heal_request_data_obj.stack_id
+            self._vnf_maintenance_plugin.project_instance_pre(context,
+                                                              vnf_dict)
             self.mgmt_update_pre(context, vnf_dict)
             self._vnf_manager.invoke(
                 driver_name, 'heal_vdu', plugin=self,
@@ -551,17 +627,17 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
             with excutils.save_and_reraise_exception():
                 vnf_dict['status'] = constants.ERROR
                 self._vnf_monitor.delete_hosting_vnf(vnf_id)
-                self.set_vnf_error_status_reason(context,
-                                                 vnf_dict['id'],
-                                                 six.text_type(e))
+                self.set_vnf_error_status_reason(context, vnf_dict['id'],
+                                                 str(e))
                 self.mgmt_update_post(context, vnf_dict)
+                self._vnf_maintenance_plugin.post(context, vnf_dict)
                 self._update_vnf_post(context, vnf_id,
                                       constants.ERROR,
                                       vnf_dict, constants.PENDING_HEAL,
                                       constants.RES_EVT_HEAL)
 
-        self.spawn_n(self._update_vnf_wait, context, vnf_dict, vim_auth,
-                     driver_name, vnf_heal=True)
+        self.spawn_n(self._heal_vnf_wait, context, vnf_dict, vim_auth,
+                     driver_name)
 
         return vnf_dict
 
@@ -583,16 +659,20 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
             except Exception as e_:
                 e = e_
                 vnf_dict['status'] = constants.ERROR
-                vnf_dict['error_reason'] = six.text_type(e)
+                vnf_dict['error_reason'] = str(e)
                 LOG.exception('_delete_vnf_wait')
                 self.set_vnf_error_status_reason(context, vnf_dict['id'],
                                                  vnf_dict['error_reason'])
 
+        self._vnf_maintenance_plugin.delete_vnf_constraints(self, context,
+                                                            vnf_dict)
         self.mgmt_delete_post(context, vnf_dict)
         self._delete_vnf_post(context, vnf_dict, e)
 
-    def delete_vnf(self, context, vnf_id):
-        vnf_dict = self._delete_vnf_pre(context, vnf_id)
+    def _delete_vnf(self, context, vnf_id, force_delete=False):
+
+        vnf_dict = self._delete_vnf_pre(context, vnf_id,
+                                        force_delete=force_delete)
         driver_name, vim_auth = self._get_infra_driver(context, vnf_dict)
         self._vnf_monitor.delete_hosting_vnf(vnf_id)
         instance_id = self._instance_id(vnf_dict)
@@ -603,6 +683,8 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
             mgmt_constants.KEY_KWARGS: {'vnf': vnf_dict},
         }
         try:
+            self._vnf_maintenance_plugin.project_instance_pre(context,
+                                                              vnf_dict)
             self.mgmt_delete_pre(context, vnf_dict)
             self.mgmt_call(context, vnf_dict, kwargs)
             if instance_id:
@@ -614,26 +696,44 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
                                          auth_attr=vim_auth,
                                          region_name=region_name)
         except Exception as e:
-            # TODO(yamahata): when the devaice is already deleted. mask
+            # TODO(yamahata): when the device is already deleted. mask
             # the error, and delete row in db
             # Other case mark error
             with excutils.save_and_reraise_exception():
-                vnf_dict['status'] = constants.ERROR
-                vnf_dict['error_reason'] = six.text_type(e)
-                self.set_vnf_error_status_reason(context, vnf_dict['id'],
-                                                 vnf_dict['error_reason'])
-                self.mgmt_delete_post(context, vnf_dict)
-                self._delete_vnf_post(context, vnf_dict, e)
+                if not force_delete:
+                    vnf_dict['status'] = constants.ERROR
+                    vnf_dict['error_reason'] = str(e)
+                    self.set_vnf_error_status_reason(context, vnf_dict['id'],
+                                                     vnf_dict['error_reason'])
+                    self.mgmt_delete_post(context, vnf_dict)
+                    self._delete_vnf_post(context, vnf_dict, e)
 
-        self.spawn_n(self._delete_vnf_wait, context, vnf_dict, vim_auth,
-                     driver_name)
+        if force_delete:
+            self._delete_vnf_force(context, vnf_dict['id'])
+            self.mgmt_delete_post(context, vnf_dict)
+            self._delete_vnf_post(context, vnf_dict, None, force_delete=True)
+        else:
+            self.spawn_n(self._delete_vnf_wait, context, vnf_dict, vim_auth,
+                         driver_name)
+
+    def delete_vnf(self, context, vnf_id, vnf=None):
+
+        # Extract "force_delete" from request's body
+        force_delete = False
+        if vnf and vnf['vnf'].get('attributes').get('force'):
+            force_delete = vnf['vnf'].get('attributes').get('force')
+        if force_delete and not context.is_admin:
+            LOG.warning("force delete is admin only operation")
+            raise exceptions.AdminRequired(reason="Admin only operation")
+
+        self._delete_vnf(context, vnf_id, force_delete=force_delete)
 
     def _handle_vnf_scaling(self, context, policy):
         # validate
         def _validate_scaling_policy():
             type = policy['type']
 
-            if type not in constants.POLICY_ACTIONS.keys():
+            if type not in constants.POLICY_ACTIONS:
                 raise exceptions.VnfPolicyTypeInvalid(
                     type=type,
                     valid_types=constants.POLICY_ACTIONS.keys(),
@@ -668,6 +768,8 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
             LOG.debug("Policy %(policy)s vnf is at %(status)s",
                       {'policy': policy['name'],
                        'status': status})
+            self._vnf_maintenance_plugin.project_instance_pre(context,
+                                                              result)
             return result
 
         # post
@@ -681,6 +783,10 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
             LOG.debug("Policy %(policy)s vnf is at %(status)s",
                       {'policy': policy['name'],
                        'status': new_status})
+            action = 'delete' if policy['action'] == 'in' else 'update'
+            self._vnf_maintenance_plugin.update_vnf_instances(self, context,
+                                                              result,
+                                                              action=action)
             return result
 
         # action
@@ -704,9 +810,7 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
                 with excutils.save_and_reraise_exception():
                     vnf['status'] = constants.ERROR
                     self.set_vnf_error_status_reason(
-                        context,
-                        policy['vnf']['id'],
-                        six.text_type(e))
+                        context, policy['vnf']['id'], str(e))
                     _handle_vnf_scaling_post(constants.ERROR)
 
         # wait
@@ -733,9 +837,7 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
                           policy['name'])
                 with excutils.save_and_reraise_exception():
                     self.set_vnf_error_status_reason(
-                        context,
-                        policy['vnf']['id'],
-                        six.text_type(e))
+                        context, policy['vnf']['id'], str(e))
                     _handle_vnf_scaling_post(constants.ERROR)
 
         _validate_scaling_policy()
@@ -971,3 +1073,20 @@ class VNFMPlugin(vnfm_db.VNFMPluginDb, VNFMMgmtMixin):
         else:
             raise vnfm.VNFInactive(vnf_id=vnf_id,
                                    message=_(' Cannot fetch details'))
+
+    def create_vnf_maintenance(self, context, vnf_id, maintenance):
+        _maintenance = self._vnf_maintenance_plugin.validate_maintenance(
+            maintenance.copy())
+        vnf = self.get_vnf(context, vnf_id)
+        _maintenance['vnf'] = vnf
+        self._vnf_maintenance_plugin.handle_maintenance(
+            self, context, _maintenance)
+        policy_action = _maintenance.get('policy_action', '')
+        if policy_action:
+            self._vnf_action.invoke(
+                policy_action['action'], 'execute_action', plugin=self,
+                context=context, vnf_dict=vnf, args=policy_action['args'])
+        else:
+            self._vnf_maintenance_plugin.request(self, context, vnf,
+                                                 _maintenance)
+        return maintenance['maintenance']
